@@ -7,7 +7,10 @@ use 5.022;
 use strict;
 use warnings;
 
-use App::PTP::Files qw(write_side_output read_side_input write_handle);
+no warnings 'experimental::smartmatch';
+use feature 'switch';
+
+use App::PTP::Files qw(write_side_output read_side_input);
 use App::PTP::PerlEnv;
 use App::PTP::Util;
 use Cwd qw(abs_path);
@@ -22,7 +25,7 @@ my @all_cmd =
     qw(prepare_perl_env do_grep do_substitute do_perl
     do_execute do_load do_sort do_list_op do_tail do_head do_delete_marked
     do_insert_marked do_set_markers do_number_lines do_file_name do_line_count
-    do_cut do_paste do_pivot do_tee do_shell do_eat);
+    do_cut do_paste do_pivot do_tee);
 our @EXPORT_OK = (@all_cmd, 'warn_or_die_if_needed');
 our %EXPORT_TAGS = (CMD => \@all_cmd);
 
@@ -193,7 +196,7 @@ sub quote_for_re {
     return quotemeta($text);
   } else {
     # We quote just the '{' or '}' characters.
-    return $text =~ s/(\{|\})/\\$1/gr;
+    return $text =~ s/({|})/\\$1/gr;
   }
 }
 
@@ -210,24 +213,12 @@ sub prepare_re2 {
   return ($use_statement, "{${re}}");
 }
 
-# This interpolate str in the Perl env (unless -Q is in effect).
-sub maybe_interpolate {
-  my ($str, $modes, $options, $command) = @_;
-  if (not $modes->{quote_regex}) {
-    $str = eval_in_safe_env("<<\"PTP_EOF_WORD\"\n${str}\nPTP_EOF_WORD\n", $options);
-    die "FATAL: Cannot eval string for --${command}: ${@}\n" if $@;
-    chomp($str);
-  }
-  return $str;
-}
-
 sub do_grep {
   my ($content, $markers, $modes, $options, $re) = @_;
   my ($use_stmt, $quoted_re) = prepare_re2($re, $modes);
   print "\$re = ${quoted_re}\n" if $options->{debug_mode};
-  my $conjuction = $modes->{inverse_match} ? 'if' : 'unless';
   my $wrapped = get_code_in_safe_env(
-      "{; ${use_stmt} undef \$_ ${conjuction} m ${quoted_re} }", $options, '--grep');
+      "{; ${use_stmt} undef \$_ unless m ${quoted_re} }", $options, '--grep');
   $. = 0;
   map { $m_setter->set(\$markers->[$.]);
         $n_setter->set($.++);
@@ -312,12 +303,10 @@ sub do_perl {
           my $r = eval { $wrapped_code->() };
           # We can't use return as we're not in a sub.
           if (warn_or_die_if_needed("Perl code failed in ${scmd}", $modes)) {
-            if ($cmd eq 'filter') {
-              1;
-            } elsif ($cmd eq 'n') {
-              $input;
-            } elsif ($cmd eq 'mark-line') {
-              $markers[$.];
+            given ($cmd) {
+              1 when 'filter';
+              $input when 'n';
+              $markers[$.] when 'mark-line';
             }
           } else {
             $r;
@@ -347,11 +336,7 @@ sub do_perl {
     if (not defined $content->[$i]) {
       undef $markers->[$i];
     } elsif (not defined $markers->[$i]) {
-      # We don't want undef here, as we will filter on it.
-      # Note that the $m and @m variables shared with the environment try to
-      # prevent the user from setting undef in them. But we don't trust that
-      # Too much.
-      $markers->[$i] = '';
+      $markers->[$i] = '';  # We don't want undef here, as we will filter on it.
     }
   }
   @$content = grep { defined } @$content;
@@ -359,13 +344,11 @@ sub do_perl {
 }
 
 sub do_execute {
-  my ($content, $markers, $modes, $options, $cmd, $code) = @_;
-  $code = "use $code;" if $cmd eq 'M';
+  my ($content, $markers, $modes, $options, $code) = @_;
   eval_in_safe_env($code, $options);
   if ($@) {
     chomp($@);
-    my $scmd = '-'.($cmd =~ s/^(..)/-$1/r); # --execute or -M.
-    die "FATAL: Perl code failed in ${scmd}: ${@}\n";
+    die "FATAL: Perl code failed in --execute: ${@}\n";
   }
 }
 
@@ -400,11 +383,11 @@ sub do_sort {
       use locale;
       @$content = sort { $a cmp $b } @$content;
     } else {
-      die sprintf "INTERNAL ERROR: Invalid comparator (%s)\n", 
+      die sprintf "INTERNAL ERROR: Invalid comparator (%s)\n.", 
                   ${$modes->{comparator}};
     }
   } else {
-    die sprintf "INTERNAL ERROR: Invalid comparator type (%s)\n",
+    die sprintf "INTERNAL ERROR: Invalid comparator type (%s)\n.",
                 Dumper($modes->{comparator}) if ref $modes->{comparator};
     my $cmp = $modes->{comparator};
     my $sort = get_code_in_safe_env("sort { $cmp } \@_", $options,
@@ -416,16 +399,11 @@ sub do_sort {
 
 sub do_list_op {
   my ($content, $markers, $modes, $options, $sub, $apply_on_markers) = @_;
-  if ($apply_on_markers eq 'none') {
-    @$content = &$sub(@$content);
-    @$markers = (0) x scalar(@$content);
-  } elsif ($apply_on_markers eq 'same') {
-    @$content = &$sub(@$content);
+  @$content = &$sub(@$content);
+  if ($apply_on_markers) {
     @$markers = &$sub(@$markers);
-  } elsif ($apply_on_markers eq 'together' ) {
-    &$sub($content, $markers);
   } else {
-    die "INTERNAL ERROR: Invalid value for \$apply_on_markers passed to do_list_op: $apply_on_markers\n";
+    @$markers = (0) x scalar(@$content);
   }
 }
 
@@ -522,11 +500,8 @@ sub do_file_name {
         ."\n";
   }
   if ($replace_all) {
-    # Does nothing to empty file.
-    if (@$content) {
-      @$content = ($name);
-      @$markers = (0);
-    }
+    @$content = ($name);
+    @$markers = (0);
   } else {
     unshift @$content, $name;
     unshift @$markers, 0;
@@ -571,11 +546,10 @@ sub do_paste {
 }
 
 sub do_pivot {
-  my ($content, $markers, $modes, $options, $action) = @_;
+  my ($content, $markers, $modes, $options, $transpose) = @_;
   my $m = 0;
-  # This is unused by the 'pivot' action, but it's not a huge issue.
-  my $re = prepare_re($modes->{input_field}, $modes);
-  if ($action eq 'transpose') {
+  if ($transpose) {
+    my $re = prepare_re($modes->{input_field}, $modes);
     $m = 0;
     my @lines =
         map { my $r = [split $re]; $m = max($m, scalar(@$r)); $r } @$content;
@@ -584,45 +558,23 @@ sub do_pivot {
           my $c = $_;
           join $modes->{output_field}, map { $_->[$c] // '' } @lines;
       } 0..($m - 1);
-  } elsif ($action eq 'pivot') {
+  } else {
     $m = 1;
     @$content = (join $modes->{output_field}, @$content);
-  } elsif ($action eq 'anti-pivot') {
-    @$content = map { split $re } @$content;
-    $m = @$content;
-  } else {
-    die "INTERNAL ERROR: unknown action for the pivot method: ${action}\n";
   }
   @$markers = (0) x $m;
 }
 
 sub do_tee {
   my ($content, $markers, $modes, $options, $file_name) = @_;
-  $file_name = maybe_interpolate($file_name, $modes, $options, 'tee');
+  if (not $modes->{quote_regex}) {
+    $file_name = eval_in_safe_env("\"${file_name}\"", $options);
+    die "FATAL: Cannot eval string for --tee: ${@}" if $@;
+  }
   # This missing_final_separator is not really an option, it is added in the
   # modes struct by the 'process' method, specifically for this function.
   write_side_output($file_name, $content, $modes->{missing_final_separator},
                     $options);
 }
 
-sub do_shell {
-  my ($content, $markers, $modes, $options, $command, $arg) = @_;
-  die "INTERNAL ERROR: Unexpected command in do_tee: ${command}\n" unless $command eq 'shell';
-  $arg = maybe_interpolate($arg, $modes, $options, $command);
-  {
-    local $SIG{PIPE} = "IGNORE";
-    open(my $pipe, '|-', $arg) or die "FATAL: Cannot execute command given to --${command}: $!\n";
-    write_handle($pipe, $content, $modes->{missing_final_separator}, $options);
-    # When run by CPAN testers, this fails sometime for unknown reason. So this
-    # is only a warning and not a fatal error.
-    close $pipe or print "WARNING: Cannot close pipe for command given to --${command}: $!\n";
-  }  
-}
-
-sub do_eat {
-  my ($content, $markers, $modes, $options) = @_;
-  @$content = ();
-  @$markers = ();
-}
-
 1;
